/**********************************************
*              rcpulse.c
*
* Simple interrupt driven RC Servo pulse generator
*
* Author: Michael Pearce <mike@kiwacan.co.nz>
*
* Started: 29 September 2006
*
* Copyright 2006 Michael Pearce All Rights reserved 
*
* Free to use in Non Commercial private projects.
* For Commercial licencing contact the author.
***********************************************
*              Version Info
***********************************************
* V1.0 - 29 September 2006
* First Draft
**********************************************/
#include "rcpulse.h"

int RC_Pulse_Done;

/**************************************
*           RCPulse_Init
*
* Setup Port pin, Timers and interrupt
*
**************************************/
void RCPulse_Init(void)
{
 T1CON=0x0000;   /* Internal OSC, Prescale=1:1 */
 RCLat=0;        /* Pin low to start with */
 RCTris=0;       /* Pin to Output */

 IPC0bits.T1IP=4;  /* Priority of 4 */
 IFS0bits.T1IF=0;   /* Clear Flag */
 IEC0bits.T1IE=1;   /* Enable Interrupt */

 RC_Pulse_Done=1;
}

/**************************************
*           RCPulse
*
* Start a RC Pulse
*
* Returns 0 if a pulse is currently active
* Returns 1 if sucessfuly started a pulse
**************************************/
int RCPulse(unsigned int Pulse)
{
 unsigned int Load;

 /* Check if already outputing a pulse */
 if(RC_Pulse_Done==0)return(0);

 /* Check Value input Limit */
 if(Pulse > RCMax) Pulse=RCMax;

 /* Calculate Timer compare value */ 
 Load=(RCBase * RCMax) + (RCBase * Pulse);
 
 TMR1=0;          /* Clear the Timer */
 PR1=Load;        /* Load the compare value */
 RCLat=1;         /* Start the Pulse */
 T1CONbits.TON=1; /* Start the Timer */

 RC_Pulse_Done=0;  /* Indicate Output is active! */

 return(1);
}

/*************************************
*           RCDone
*
* Indicates Pulse State
*
* Returns 1 for Done
* Returns 0 for pulse active
*************************************/
int RCDone(void)
{
 return(RC_Pulse_Done);
}

/*************************************
*           _T1Interrupt
*
* Interrupt to end the pulse
*************************************/
void _ISRFAST _T1Interrupt(void)
{
 RCLat=0;           /* Turn Pin Off   */
 T1CONbits.TON=0;   /* Turn Timer Off */
 IFS0bits.T1IF=0;   /* Clear Interrupt flag */
 RC_Pulse_Done=1;   /* Indicate pulse is complete */
}



